import * as fs from 'fs';
import * as path from 'path';
import * as util from 'util';
export class Reader {
    constructor() {
        this.foldersToExlcude = ['node_modules', 'dist', '.git'];
        this.extensionToRead = '.feature';
        this.readDirAsync = util.promisify(fs.readdir);
        this.statAsync = util.promisify(fs.stat);
        this.readFileAsync = util.promisify(fs.readFile);
    }
    async readFeatureFilesFromFolder(folder, onReadFilesFromFolder) {
        let results = [];
        try {
            const readFilesList = await this.readDirAsync(folder);
            let totalFilesToRead = readFilesList.length;
            if (!totalFilesToRead) {
                return onReadFilesFromFolder(null, results);
            }
            readFilesList
                .map(async (baseNameReadFile) => {
                let readFile = baseNameReadFile;
                readFile = path.resolve(folder, readFile);
                const stat = await this.statAsync(readFile);
                if (stat && stat.isDirectory()) {
                    this.readFeatureFilesFromFolder(readFile, (_, readData) => {
                        if (!this.foldersToExlcude.includes(baseNameReadFile)) {
                            results = results.concat(readData);
                        }
                        if (!--totalFilesToRead) {
                            onReadFilesFromFolder(null, results);
                        }
                    });
                }
                else {
                    if (readFile.indexOf(this.extensionToRead) >= 0) {
                        results.push(readFile);
                    }
                    if (!--totalFilesToRead) {
                        onReadFilesFromFolder(null, results);
                    }
                }
            });
        }
        catch (readFeatureFilesFromFolderError) {
            if (readFeatureFilesFromFolderError instanceof Error) {
                onReadFilesFromFolder(readFeatureFilesFromFolderError, []);
            }
            else {
                onReadFilesFromFolder(new Error('Unknown error in readFeatureFilesFromFolder'), []);
            }
        }
    }
    async readContentFeatureFile(file) {
        try {
            return await this.readFileAsync(file, { encoding: 'utf8' });
        }
        catch {
            return '';
        }
    }
    getRowsFeatureFile(contentFile) {
        let rowsFile = contentFile.split('\n');
        rowsFile = rowsFile
            .map(row => row.trim())
            .filter(row => !(row === ''));
        return rowsFile;
    }
}
//# sourceMappingURL=reader.js.map